@@ -1,33 +1,21 @@
-/* $Id: cache.c 2424 2010-03-16 05:33:03Z aturner $ */
+/* $Id$ */
 
 /*
- * Copyright (c) 2001-2010 Aaron Turner.
- * All rights reserved.
+ *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
+ *   Copyright (c) 2013-2014 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *   The Tcpreplay Suite of tools is free software: you can redistribute it 
+ *   and/or modify it under the terms of the GNU General Public License as 
+ *   published by the Free Software Foundation, either version 3 of the 
+ *   License, or with the authors permission any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the names of the copyright owners nor the names of its
- *    contributors may be used to endorse or promote products derived from
- *    this software without specific prior written permission.
+ *   The Tcpreplay Suite is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
- * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *   You should have received a copy of the GNU General Public License
+ *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -102,7 +90,7 @@ read_cache(char **cachedata, const char *cachefile, char **comment)
 
     /* verify version */
     if (atoi(header.version) != atoi(CACHEVERSION))
-        errx(-1, "Unable to process %s: cache file version missmatch",
+        errx(-1, "Unable to process %s: cache file version mismatch",
              cachefile);
 
     /* read the comment */
@@ -124,11 +112,11 @@ read_cache(char **cachedata, const char *cachefile, char **comment)
     header.packets_per_byte = ntohs(header.packets_per_byte);    
     cache_size = header.num_packets / header.packets_per_byte;
         
-    /* deal with any remainder, becuase above divsion is integer */
+    /* deal with any remainder, because above division is integer */
     if (header.num_packets % header.packets_per_byte)
       cache_size ++;
 
-    dbgx(1, "Cache file contains %llu packets in %llu bytes",
+    dbgx(1, "Cache file contains %" PRIu64 " packets in " COUNTER_SPEC " bytes",
         header.num_packets, cache_size);
 
     dbgx(1, "Cache uses %d packets per byte", header.packets_per_byte);
@@ -138,10 +126,10 @@ read_cache(char **cachedata, const char *cachefile, char **comment)
     /* read in the cache */
     if ((COUNTER)(read_size = read(cachefd, *cachedata, cache_size)) 
             != cache_size)
-        errx(-1, "Cache data length (%ld bytes) doesn't match "
+        errx(-1, "Cache data length (%zu bytes) doesn't match "
             "cache header (" COUNTER_SPEC " bytes)", read_size, cache_size);
 
-    dbgx(1, "Loaded in %llu packets from cache.", header.num_packets);
+    dbgx(1, "Loaded in %" PRIu64 " packets from cache.", header.num_packets);
 
     close(cachefd);
     return (header.num_packets);
@@ -159,11 +147,10 @@ write_cache(tcpr_cache_t * cachedata, const int out_file, COUNTER numpackets,
 {
     tcpr_cache_t *mycache = NULL;
     tcpr_cache_file_hdr_t *cache_header = NULL;
-    u_int32_t chars, last = 0;
+    uint32_t chars, last = 0;
     COUNTER packets = 0;
     ssize_t written = 0;
 
-    assert(cachedata);
     assert(out_file);
 
     /* write a header to our file */
@@ -176,7 +163,7 @@ write_cache(tcpr_cache_t * cachedata, const int out_file, COUNTER numpackets,
 
     /* we can't strlen(NULL) so ... */
     if (comment != NULL) {
-        cache_header->comment_len = htons((u_int16_t)strlen(comment));
+        cache_header->comment_len = htons((uint16_t)strlen(comment));
     } else {
         cache_header->comment_len = 0;
     }
@@ -200,36 +187,38 @@ write_cache(tcpr_cache_t * cachedata, const int out_file, COUNTER numpackets,
                  written == -1 ? strerror(errno) : "");
     }
 
-    mycache = cachedata;
+    if (cachedata) {
+    	mycache = cachedata;
 
-    while (!last) {
-        /* increment total packets */
-        packets += mycache->packets;
+    	while (!last) {
+    		/* increment total packets */
+    		packets += mycache->packets;
 
-        /* calculate how many chars to write */
-        chars = mycache->packets / CACHE_PACKETS_PER_BYTE;
-        if (mycache->packets % CACHE_PACKETS_PER_BYTE) {
-            chars++;
-            dbgx(1, "Bumping up to the next byte: %d %% %d", mycache->packets,
-                CACHE_PACKETS_PER_BYTE);
-        }
+    		/* calculate how many chars to write */
+    		chars = mycache->packets / CACHE_PACKETS_PER_BYTE;
+    		if (mycache->packets % CACHE_PACKETS_PER_BYTE) {
+    			chars++;
+    			dbgx(1, "Bumping up to the next byte: %d %% %d", mycache->packets,
+    					CACHE_PACKETS_PER_BYTE);
+    		}
 
-        /* write to file, and verify it wrote properly */
-        written = write(out_file, mycache->data, chars);
-        dbgx(1, "Wrote %zu bytes of cache data", written);
-        if (written != (ssize_t)chars)
-            errx(-1, "Only wrote %zu of %i bytes to cache file!", written, chars);
+    		/* write to file, and verify it wrote properly */
+    		written = write(out_file, mycache->data, chars);
+    		dbgx(1, "Wrote %zu bytes of cache data", written);
+    		if (written != (ssize_t)chars)
+    			errx(-1, "Only wrote %zu of %i bytes to cache file!", written, chars);
 
-        /*
-         * if that was the last, stop processing, otherwise wash,
-         * rinse, repeat
-         */
-        if (mycache->next != NULL) {
-            mycache = mycache->next;
-        }
-        else {
-            last = 1;
-        }
+    		/*
+    		 * if that was the last, stop processing, otherwise wash,
+    		 * rinse, repeat
+    		 */
+    		if (mycache->next != NULL) {
+    			mycache = mycache->next;
+    		}
+    		else {
+    			last = 1;
+    		}
+    	}
     }
     /* return number of packets written */
     return (packets);
@@ -258,7 +247,7 @@ add_cache(tcpr_cache_t ** cachedata, const int send, const tcpr_dir_t interface)
 {
     static tcpr_cache_t *lastcache = NULL;
     u_char *byte = NULL;
-    u_int32_t bit;
+    uint32_t bit;
     tcpr_dir_t result = TCPR_DIR_ERROR;
     COUNTER index;
 #ifdef DEBUG
@@ -337,7 +326,7 @@ tcpr_dir_t
 check_cache(char *cachedata, COUNTER packetid)
 {
     COUNTER index = 0;
-    u_int32_t bit;
+    uint32_t bit;
 
     assert(cachedata);
 
@@ -345,12 +334,12 @@ check_cache(char *cachedata, COUNTER packetid)
         err(-1, "packetid must be > 0");
 
     index = (packetid - 1) / (COUNTER)CACHE_PACKETS_PER_BYTE;
-    bit = (u_int32_t)(((packetid - 1) % (COUNTER)CACHE_PACKETS_PER_BYTE) * 
+    bit = (uint32_t)(((packetid - 1) % (COUNTER)CACHE_PACKETS_PER_BYTE) * 
         (COUNTER)CACHE_BITS_PER_PACKET) + 1;
 
 #ifdef DEBUG
     dbgx(3, "Index: " COUNTER_SPEC "\tBit: %d\tByte: %hhu\tMask: %hhu", index, bit,
-        cachedata[index], (cachedata[index] & (char)(1 << bit)));
+        cachedata[index], (uint8_t)(cachedata[index] & (char)(1 << bit)));
 #endif
 
     if (!(cachedata[index] & (char)(1 << bit))) {
@@ -368,12 +357,3 @@ check_cache(char *cachedata, COUNTER packetid)
 
     return TCPR_DIR_ERROR;
 }
-
-/*
- Local Variables:
- mode:c
- indent-tabs-mode:nil
- c-basic-offset:4
- End:
-*/
-
