@@ -1,40 +1,29 @@
-/* $Id: cidr.c 2423 2010-03-13 07:09:49Z aturner $ */
+/* $Id$ */
 
 /*
- * Copyright (c) 2001-2010 Aaron Turner.
- * All rights reserved.
+ *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
+ *   Copyright (c) 2013-2014 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *   The Tcpreplay Suite of tools is free software: you can redistribute it 
+ *   and/or modify it under the terms of the GNU General Public License as 
+ *   published by the Free Software Foundation, either version 3 of the 
+ *   License, or with the authors permission any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the names of the copyright owners nor the names of its
- *    contributors may be used to endorse or promote products derived from
- *    this software without specific prior written permission.
+ *   The Tcpreplay Suite is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
- * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *   You should have received a copy of the GNU General Public License
+ *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 #include "config.h"
 #include "defines.h"
 #include "common.h"
+#ifndef HAVE_STRLCPY
 #include "lib/strlcpy.h"
+#endif 
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -85,11 +74,12 @@ void
 destroy_cidr(tcpr_cidr_t * cidr)
 {
 
-    if (cidr != NULL)
+    if (cidr != NULL) {
         if (cidr->next != NULL)
             destroy_cidr(cidr->next);
 
-    safe_free(cidr);
+        safe_free(cidr);
+    }
     return;
 
 }
@@ -277,7 +267,7 @@ cidr2cidr(char *cidr)
     /* we only get here on error parsing input */
 error:
     memset(ebuf, '\0', EBUF_SIZE);
-    strcpy(ebuf, "Unable to parse as a vaild CIDR: ");
+    strcpy(ebuf, "Unable to parse as a valid CIDR: ");
     strlcat(ebuf, cidr, EBUF_SIZE);
     errx(-1, "%s", ebuf);
     return NULL;
@@ -495,7 +485,7 @@ ip_in_cidr(const tcpr_cidr_t * mycidr, const unsigned long ip)
 
 #ifdef DEBUG
     /* copy this for debug purposes, since it's not re-entrant */
-    strlcpy(netstr, get_addr2name4(htonl(mycidr->u.network), RESOLVE), 20);
+    strlcpy(netstr, get_addr2name4(mycidr->u.network, RESOLVE), 20);
 #endif
 
     /* if they're the same, then ip is in network */
@@ -667,19 +657,20 @@ char *
 cidr2iplist(tcpr_cidr_t * cidr, char delim)
 {
     char *list = NULL;
-    char ipaddr[16];
+    char ipaddr[18], tempbuff[20];
     u_int32_t size, addr, first, last, numips;
     struct in_addr in;
+    int i;
 
     /* 
      * 16 bytes per IP + delim
      * # of IP's = 2^(32-masklen)
      */
     numips = 2;
-    for (int i = 2; i <= (32 - cidr->masklen); i++)
+    for (i = 2; i <= (32 - cidr->masklen); i++)
         numips *= 2;
 
-    size = 16 * numips;
+    size = 17 * numips - 1;
 
     list = (char *)safe_malloc(size);
 
@@ -694,14 +685,15 @@ cidr2iplist(tcpr_cidr_t * cidr, char delim)
     /* loop through all but the last one */
     for (addr = first; addr < last; addr++) {
         in.s_addr = htonl(addr);
-        snprintf(ipaddr, 17, "%s%c", inet_ntoa(in), delim);
+        snprintf(tempbuff, 17, "%s%c", inet_ntoa(in), delim);
+        memcpy(ipaddr, tempbuff, 16);
         dbgx(2, "%s", ipaddr);
         strlcat(list, ipaddr, size);
     }
 
     /* last is a special case, end in \0 */
     in.s_addr = htonl(addr);
-    snprintf(ipaddr, 16, "%s", inet_ntoa(in));
+    snprintf(ipaddr, 17, "%s", inet_ntoa(in));
     strlcat(list, ipaddr, size);
 
     return list;
