@@ -1,4 +1,5 @@
-/* $Id: fakepoll.h 2423 2010-03-13 07:09:49Z aturner $ */
+/* $Id$ */
+
 /*
  * fakepoll.h
  *
@@ -74,13 +75,3 @@ int poll(struct pollfd *ufds, unsigned int nfds, int timeout);
 #endif
 #endif
 #endif
-
-/*
- Local Variables:
- mode:c
- indent-tabs-mode:nil
- c-basic-offset:4
- End:
-*/
-
-
