@@ -1,33 +1,21 @@
-/* $Id: tcpdump.c 2423 2010-03-13 07:09:49Z aturner $ */
+/* $Id$ */
 
 /*
- * Copyright (c) 2001-2010 Aaron Turner.
- * All rights reserved.
+ *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
+ *   Copyright (c) 2013-2014 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *   The Tcpreplay Suite of tools is free software: you can redistribute it 
+ *   and/or modify it under the terms of the GNU General Public License as 
+ *   published by the Free Software Foundation, either version 3 of the 
+ *   License, or with the authors permission any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the names of the copyright owners nor the names of its
- *    contributors may be used to endorse or promote products derived from
- *    this software without specific prior written permission.
+ *   The Tcpreplay Suite is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
- * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *   You should have received a copy of the GNU General Public License
+ *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -48,6 +36,7 @@
 #include "defines.h"
 #include "common.h"
 
+#include <stdlib.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <sys/socket.h>
@@ -59,7 +48,10 @@
 #endif
 
 #include "tcpdump.h"
+
+#ifndef HAVE_STRLCPY
 #include "lib/strlcpy.h"
+#endif
 
 #ifdef DEBUG
 extern int debug;
@@ -153,7 +145,7 @@ tcpdump_open(tcpdump_t *tcpdump, pcap_t *pcap)
 {
     int infd[2], outfd[2];
     FILE *writer;
-    
+
     assert(tcpdump);
     assert(pcap);
 
@@ -166,7 +158,7 @@ tcpdump_open(tcpdump_t *tcpdump, pcap_t *pcap)
     if (! can_exec(TCPDUMP_BINARY)) {
         errx(-1, "Unable to execute tcpdump binary: %s", TCPDUMP_BINARY);
     }
-    
+
 #ifdef DEBUG
      strlcpy(tcpdump->debugfile, TCPDUMP_DEBUG, sizeof(tcpdump->debugfile));
      if (debug >= 5) {
@@ -192,13 +184,12 @@ tcpdump_open(tcpdump_t *tcpdump, pcap_t *pcap)
     /* create our socket pair to read packet decode from tcpdump */
     if (socketpair(AF_UNIX, SOCK_STREAM, 0, outfd) < 0)
         errx(-1, "Unable to create stdout socket pair: %s", strerror(errno));
- 
-         
+
     if ((tcpdump->pid = fork() ) < 0)
         errx(-1, "Fork failed: %s", strerror(errno));
 
     dbgx(2, "tcpdump pid: %d", tcpdump->pid);
-    
+
     if (tcpdump->pid > 0) {
         /* we're still in tcpreplay */
         dbgx(2, "[parent] closing input fd %d", infd[1]);
@@ -221,7 +212,7 @@ tcpdump_open(tcpdump_t *tcpdump, pcap_t *pcap)
 
         if (fcntl(tcpdump->outfd, F_SETFL, O_NONBLOCK) < 0)
             warnx("[parent] Unable to fnctl stdout socket:\n%s", strerror(errno));
-            
+
     }
     else {
         dbg(2, "[child] started the kid");
@@ -231,14 +222,14 @@ tcpdump_open(tcpdump_t *tcpdump, pcap_t *pcap)
         dbgx(2, "[child] closing out fd %d", outfd[0]);
         close(infd[0]); /* close the tcpreplay side */
         close(outfd[0]);
-    
+
         /* copy our side of the socketpair to our stdin */
         if (infd[1] != STDIN_FILENO) {
             if (dup2(infd[1], STDIN_FILENO) != STDIN_FILENO)
                 errx(-1, "[child] Unable to copy socket to stdin: %s", 
                     strerror(errno));
         }
-    
+
         /* copy our side of the socketpair to our stdout */
         if (outfd[1] != STDOUT_FILENO) {
             if (dup2(outfd[1], STDOUT_FILENO) != STDOUT_FILENO)
@@ -252,7 +243,7 @@ tcpdump_open(tcpdump_t *tcpdump, pcap_t *pcap)
             errx(-1, "Unable to exec tcpdump: %s", strerror(errno));
 
     }
-    
+
     return TRUE;
 }
 
