@@ -2,8 +2,79 @@
 #define __DEFINES_H__
 
 #include "config.h"
+
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#ifdef WIN32
+#include "msvc_inttypes.h"
+#include "msvc_stdint.h"
+#else
+
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
+#endif /* WIN32 */
+
+#ifdef HAVE_SYS_SOCKET
+#include <sys/socket.h>
+#endif
+
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+
+#ifdef HAVE_NETINET_IN_SYSTM_H
+#include <netinet/in_systm.h>
+#endif
+
 #include "tcpr.h"
 
+#ifdef HAVE_BPF
+#include <net/bpf.h>
+#define PCAP_DONT_INCLUDE_PCAP_BPF_H 1
+#endif
+
+#if defined INCLUDE_PCAP_BPF_H_FILE && !defined PCAP_DONT_INCLUDE_PCAP_BPF_H
+#include "@INCLUDE_PCAP_BPF_HEADER@"
+#define PCAP_DONT_INCLUDE_PCAP_BPF_H 1 /* don't re-include it in pcap.h */
+#endif
+
+#include "/usr/include/pcap.h"
+
+/* include our own strlcat/strlcpy? */
+#ifndef HAVE_STRLCPY
+#include "lib/strlcpy.h"
+#endif
+
+/*
+ * net/bpf.h doesn't include DLT types, but pcap-bpf.h does.
+ * Unfortunately, pcap-bpf.h also includes things in net/bpf.h
+ * while also missing some key things (wow, that sucks)
+ * The result is that I stole the DLT types from pcap-bpf.h and
+ * put them in here.
+ */
+#include "common/dlt_names.h"
+
+#ifdef HAVE_BOOL_H
+#include <bool.h>
+#endif
+
+#ifdef HAVE_STDBOOL_H
+#include <stdbool.h>
+#endif
+
+
 /* should packet counters be 32 or 64 bit? --enable-64bit */
 #ifdef ENABLE_64BITS
 #define COUNTER unsigned long long
@@ -13,36 +84,9 @@
 #define COUNTER_SPEC "%lu"
 #endif
 
-
-#ifdef HAVE_BPF
-#include <net/bpf.h>
-#define PCAP_DONT_INCLUDE_PCAP_BPF_H 1
-#endif
-
-#if defined INCLUDE_PCAP_BPF_H_FILE && !defined PCAP_DONT_INCLUDE_PCAP_BPF_H
-#include "/usr/local/include/pcap/bpf.h"
-#define PCAP_DONT_INCLUDE_PCAP_BPF_H 1 /* don't re-include it in pcap.h */
-#endif
-
-#include "/usr/local/include/pcap.h"
-
-#include "lib/strlcpy.h"
 #include "common/list.h"
 #include "common/cidr.h"
 
-/* 
- * net/bpf.h doesn't include DLT types, but pcap-bpf.h does.  
- * Unfortunately, pcap-bpf.h also includes things in net/bpf.h
- * while also missing some key things (wow, that sucks)
- * The result is that I stole the DLT types from pcap-bpf.h and 
- * put them in here.
- */
-#include "common/dlt_names.h"
-
-
-#ifdef HAVE_LIBNET
-#include "@LNETINC@"
-#endif
 
 typedef struct tcpr_ipv4_hdr ipv4_hdr_t;
 typedef struct tcpr_ipv6_hdr ipv6_hdr_t;
@@ -59,14 +103,13 @@ typedef struct tcpr_dnsv4_hdr dnsv4_hdr_t;
 /* our custom typdefs/structs */
 typedef u_char tcpr_macaddr_t[TCPR_ETH_H];
 
-struct tcpr_bpf_s {
+typedef struct tcpr_bpf_s {
     char *filter;
     int optimize;
     struct bpf_program program;
-};
-typedef struct tcpr_bpf_s tcpr_bpf_t;
+} tcpr_bpf_t;
 
-struct tcpr_xX_s {
+typedef struct tcpr_xX_s {
 #define xX_MODE_INCLUDE 'x'
 #define xX_MODE_EXCLUDE 'X'
     int mode;
@@ -75,18 +118,16 @@ struct tcpr_xX_s {
 #define xX_TYPE_LIST 1
 #define xX_TYPE_CIDR 2
     int type;
-};
-typedef struct tcpr_xX_s tcpr_xX_t;
+} tcpr_xX_t;
 
 /* number of ports 0-65535 */
 #define NUM_PORTS 65536
-struct tcpr_services_s {
-    char tcp[NUM_PORTS];    
+typedef struct tcpr_services_s {
+    char tcp[NUM_PORTS];
     char udp[NUM_PORTS];
-};
-typedef struct tcpr_services_s tcpr_services_t;
+} tcpr_services_t;
 
-struct tcpr_speed_s {
+typedef struct tcpr_speed_s {
     /* speed modifiers */
     int mode;
 #define SPEED_MULTIPLIER 1
@@ -94,17 +135,17 @@ struct tcpr_speed_s {
 #define SPEED_PACKETRATE 3
 #define SPEED_TOPSPEED   4
 #define SPEED_ONEATATIME 5
-    float speed;
+    COUNTER speed;
+    float multiplier;
     int pps_multi;
-};
-typedef struct tcpr_speed_s tcpr_speed_t;
+} tcpr_speed_t;
 
 #define MAX_FILES   1024        /* Max number of files we can pass to tcpreplay */
 
 #define DEFAULT_MTU 1500        /* Max Transmission Unit of standard ethernet
                                  * don't forget *frames* are MTU + L2 header! */
 
-#define MAXPACKET 65535         /* was 16436 linux loopback, but maybe something is bigger then 
+#define MAXPACKET 65535         /* was 16436 linux loopback, but maybe something is bigger then
                                    linux loopback */
 
 #define MAX_SNAPLEN 65535       /* tell libpcap to capture the entire packet */
@@ -118,42 +159,42 @@ typedef struct tcpr_speed_s tcpr_speed_t;
 
 /* HP-UX already defines TRUE/FALSE */
 #ifndef TRUE
-enum bool_t {
-	FALSE = 0,
-	TRUE
-};
+typedef enum bool_e {
+    FALSE = 0,
+    TRUE
+} bool_t;
 #endif
 
 #define EBUF_SIZE 1024           /* size of our error buffers */
 #define MAC_SIZE  7             /* size of the mac[] buffer */
 
-enum pad_t {
-	PAD_PACKET,
-	TRUNC_PACKET
-};
+typedef enum pad_e {
+    PAD_PACKET,
+    TRUNC_PACKET
+} pad_t;
 
 #define DNS_QUERY_FLAG 0x8000
 
-enum direction_t {
-	DIR_UNKNOWN = -1,
-	DIR_CLIENT = 0,
-	DIR_SERVER = 1,
-	DIR_ANY = 2
-};
+typedef enum direction_e {
+    DIR_UNKNOWN = -1,
+    DIR_CLIENT  = 0,
+    DIR_SERVER  = 1,
+    DIR_ANY     = 2
+} direction_t;
 
-enum tcpprep_mode {
-	ERROR_MODE,     /* Some kind of error has occurred */
-	CIDR_MODE,     	/* single pass, CIDR netblock */
-	REGEX_MODE,		/* single pass, regex */
-	PORT_MODE,		/* single pass, use src/dst ports to split */
-	MAC_MODE,		/* single pass, use src mac to split */
-	FIRST_MODE,     /* single pass, use first seen to split */
-	AUTO_MODE,		/* first pass through in auto mode */
-	ROUTER_MODE,	/* second pass through in router mode */
-	BRIDGE_MODE,	/* second pass through in bridge mode */
-	SERVER_MODE,	/* second pass through in server (router) mode */
-	CLIENT_MODE 	/* second pass through in client (router) mode */
-};
+typedef enum tcpprep_mode_e {
+    ERROR_MODE,     /* Some kind of error has occurred */
+    CIDR_MODE,      /* single pass, CIDR netblock */
+    REGEX_MODE,     /* single pass, regex */
+    PORT_MODE,      /* single pass, use src/dst ports to split */
+    MAC_MODE,       /* single pass, use src mac to split */
+    FIRST_MODE,     /* single pass, use first seen to split */
+    AUTO_MODE,      /* first pass through in auto mode */
+    ROUTER_MODE,    /* second pass through in router mode */
+    BRIDGE_MODE,    /* second pass through in bridge mode */
+    SERVER_MODE,    /* second pass through in server (router) mode */
+    CLIENT_MODE     /* second pass through in client (router) mode */
+} tcpprep_mode_t;
 
 #define BROADCAST_MAC "\xFF\xFF\xFF\xFF\xFF\xFF"
 
@@ -253,13 +294,14 @@ typedef u_int32_t uint32_t
 #define TIMEVAL_TO_MILLISEC(x)  (((x)->tv_sec * 1000) + ((x)->tv_usec / 1000))
 #define TIMEVAL_TO_MICROSEC(x)  (((x)->tv_sec * 1000000) + (x)->tv_usec)
 #define TIMEVAL_TO_NANOSEC(x)   ((u_int64_t)((x)->tv_sec * 1000000000) + ((u_int64_t)(x)->tv_usec * 1000))
+#define TIMSTAMP_TO_MICROSEC(x) (TIMEVAL_TO_MICROSEC(x))
 
 #define MILLISEC_TO_TIMEVAL(x, tv)          \
     do {                                    \
         (tv)->tv_sec = (x) / 1000;          \
         (tv)->tv_usec = (x * 1000) - ((tv)->tv_sec * 1000000);   \
     } while(0)
-    
+
 #define MICROSEC_TO_TIMEVAL(x, tv)          \
     do {                                    \
         (tv)->tv_sec = (x) / 1000000;       \
@@ -269,7 +311,7 @@ typedef u_int32_t uint32_t
 #define NANOSEC_TO_TIMEVAL(x, tv)           \
     do {                                    \
         (tv)->tv_sec =  (x) / 1000000000;   \
-        (tv)->tv_usec = ((x) % 1000000000) / 1000);   \
+        (tv)->tv_usec = ((x) % 1000000000) / 1000;   \
     } while(0)
 
 #define NANOSEC_TO_TIMESPEC(x, ts)          \
@@ -282,4 +324,31 @@ typedef u_int32_t uint32_t
 #define TIMESPEC_TO_MICROSEC(x)  (((x)->tv_sec * 1000000) + (x)->tv_nsec / 1000)
 #define TIMESPEC_TO_NANOSEC(x) ((u_int64_t)((x)->tv_sec * 1000000000) + ((u_int64_t)(x)->tv_nsec))
 
+/* 
+ * Help suppress some compiler warnings
+ * No problem if variable is actually used 
+*/
+#ifdef UNUSED 
+#elif defined(__GNUC__) 
+# define UNUSED(x) UNUSED_ ## x __attribute__((unused)) 
+#elif defined(__LCLINT__) 
+# define UNUSED(x) /*@unused@*/ x 
+#else 
+# define UNUSED(x) x 
+#endif
+
+#ifndef max
+#  define max(a,b) \
+       ({ __typeof__ (a) _a = (a); \
+           __typeof__ (b) _b = (b); \
+       _a > _b ? _a : _b; })
+#endif
+
+#ifndef min
+#  define min(a,b) \
+       ({ __typeof__ (a) _a = (a); \
+           __typeof__ (b) _b = (b); \
+       _a > _b ? _b : _a; })
+#endif
+
 #endif /* DEFINES */
