@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: pkt.c 2303 2009-05-06 18:48:20Z aturner $
+ * $Id$
  */
 
 #include "config.h"
@@ -257,7 +257,7 @@ pktq_reverse(struct pktq *pktq)
 		next = TAILQ_NEXT(pkt, pkt_next);
 		TAILQ_INSERT_HEAD(&tmpq, pkt, pkt_next);
 	}
-	*pktq = tmpq;
+	TAILQ_COPY(pktq, &tmpq, pkt_next);
 }
 
 void
