@@ -1,35 +1,23 @@
-/* $Id: tcpbridge.c 2423 2010-03-13 07:09:49Z aturner $ */
+/* $Id$ */
 
 /*
- * Copyright (c) 2004-2010 Aaron Turner.
- * All rights reserved.
+ *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
+ *   Copyright (c) 2013-2014 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *   The Tcpreplay Suite of tools is free software: you can redistribute it 
+ *   and/or modify it under the terms of the GNU General Public License as 
+ *   published by the Free Software Foundation, either version 3 of the 
+ *   License, or with the authors permission any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the names of the copyright owners nor the names of its
- *    contributors may be used to endorse or promote products derived from
- *    this software without specific prior written permission.
+ *   The Tcpreplay Suite is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
- * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *   You should have received a copy of the GNU General Public License
+ *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
- 
+
 /*
  * Purpose: Modify packets in a pcap file based on rules provided by the
  * user to offload work from tcpreplay and provide a easier means of 
@@ -54,16 +42,14 @@
 #include "tcpbridge_opts.h"
 #include "bridge.h"
 #include "tcpedit/tcpedit.h"
-#include "send_packets.h"
 
 #ifdef DEBUG
 int debug;
 #endif
 
 
-COUNTER bytes_sent, total_bytes, failed, pkts_sent, cache_packets;
-struct timeval begin, end;
-volatile int didsig;
+COUNTER cache_packets;
+tcpreplay_stats_t stats;
 tcpbridge_opt_t options;
 tcpedit_t *tcpedit;
 
@@ -85,20 +71,19 @@ main(int argc, char *argv[])
 
     post_args(argc, argv);
 
-   
     /* init tcpedit context */
     if (tcpedit_init(&tcpedit, pcap_datalink(options.pcap1)) < 0) {
         errx(-1, "Error initializing tcpedit: %s", tcpedit_geterr(tcpedit));
     }
-    
+
     /* parse the tcpedit args */
-    rcode = tcpedit_post_args(&tcpedit);
+    rcode = tcpedit_post_args(tcpedit);
     if (rcode < 0) {
         errx(-1, "Unable to parse args: %s", tcpedit_geterr(tcpedit));
     } else if (rcode == 1) {
         warnx("%s", tcpedit_geterr(tcpedit));
     }
-    
+
     if (tcpedit_validate(tcpedit) < 0) {
         errx(-1, "Unable to edit packets given options:\n%s",
                 tcpedit_geterr(tcpedit));
@@ -111,7 +96,7 @@ main(int argc, char *argv[])
     }
 #endif
 
-    if (gettimeofday(&begin, NULL) < 0)
+    if (gettimeofday(&stats.start_time, NULL) < 0)
         err(-1, "gettimeofday() failed");
 
 
@@ -135,16 +120,14 @@ main(int argc, char *argv[])
 void 
 init(void)
 {
-    
-    bytes_sent = total_bytes = failed = pkts_sent = cache_packets = 0;
+
+    memset(&stats, 0, sizeof(stats));
     memset(&options, 0, sizeof(options));
-    
+
     options.snaplen = 65535;
     options.promisc = 1;
     options.to_ms = 1;
 
-    total_bytes = 0;
-
     if (fcntl(STDERR_FILENO, F_SETFL, O_NONBLOCK) < 0)
         warnx("Unable to set STDERR to non-blocking: %s", strerror(errno));
 
@@ -163,7 +146,7 @@ post_args(_U_ int argc, _U_ char *argv[])
 #else
     interface_list_t *intlist = NULL;
 #endif
-    
+
 #ifdef DEBUG
     if (HAVE_OPT(DBUG))
         debug = OPT_VALUE_DBUG;
@@ -171,15 +154,14 @@ post_args(_U_ int argc, _U_ char *argv[])
     if (HAVE_OPT(DBUG))
         warn("not configured with --enable-debug.  Debugging disabled.");
 #endif
-    
+
 
 #ifdef ENABLE_VERBOSE
     if (HAVE_OPT(VERBOSE))
         options.verbose = 1;
-    
+
     if (HAVE_OPT(DECODE))
         options.tcpdump->args = safe_strdup(OPT_ARG(DECODE));
-    
 #endif
 
     if (HAVE_OPT(UNIDIR))
@@ -189,22 +171,29 @@ post_args(_U_ int argc, _U_ char *argv[])
         options.limit_send = OPT_VALUE_LIMIT; /* default is -1 */
 
 
-    if ((intname = get_interface(intlist, OPT_ARG(INTF1))) == NULL)
-        errx(-1, "Invalid interface name/alias: %s", OPT_ARG(INTF1));
-    
+    if ((intname = get_interface(intlist, OPT_ARG(INTF1))) == NULL) {
+        if (!strncmp(OPT_ARG(INTF1), "qtx:", 4))
+            errx(-1, "Unable to connect to Quick TX interface %s. Ensure Quick TX module is installed (see INSTALL).",
+                    OPT_ARG(INTF1));
+        else if (!strncmp(OPT_ARG(INTF1), "netmap:", 7) || !strncmp(OPT_ARG(INTF1), "vale", 4))
+            errx(-1, "Unable to connect to netmap interface %s. Ensure netmap module is installed (see INSTALL).",
+                    OPT_ARG(INTF1));
+        else
+            errx(-1, "Invalid interface name/alias: %s", OPT_ARG(INTF1));
+    }
+
     options.intf1 = safe_strdup(intname);
 
     if (HAVE_OPT(INTF2)) {
         if ((intname = get_interface(intlist, OPT_ARG(INTF2))) == NULL)
             errx(-1, "Invalid interface name/alias: %s", OPT_ARG(INTF2));
-    
+
         options.intf2 = safe_strdup(intname);
     }
-    
 
     if (HAVE_OPT(MAC)) {
         int ct = STACKCT_OPT(MAC);
-        char **list = STACKLST_OPT(MAC);
+        char **list = (char**)STACKLST_OPT(MAC);
         int first = 1;
         do {
             char *p = *list++;
@@ -221,7 +210,7 @@ post_args(_U_ int argc, _U_ char *argv[])
      * if user doesn't specify MAC address on CLI, query for it 
      */
     if (memcmp(options.intf1_mac, "\00\00\00\00\00\00", ETHER_ADDR_LEN) == 0) {
-        if ((sp = sendpacket_open(options.intf1, ebuf, TCPR_DIR_C2S)) == NULL)
+        if ((sp = sendpacket_open(options.intf1, ebuf, TCPR_DIR_C2S, SP_TYPE_NONE, NULL)) == NULL)
             errx(-1, "Unable to open interface %s: %s", options.intf1, ebuf);
 
         if ((eth_buff = sendpacket_get_hwaddr(sp)) == NULL) {
@@ -233,7 +222,7 @@ post_args(_U_ int argc, _U_ char *argv[])
     }
 
     if (memcmp(options.intf2_mac, "\00\00\00\00\00\00", ETHER_ADDR_LEN) == 0) {
-        if ((sp = sendpacket_open(options.intf2, ebuf, TCPR_DIR_S2C)) == NULL)
+        if ((sp = sendpacket_open(options.intf2, ebuf, TCPR_DIR_S2C, SP_TYPE_NONE, NULL)) == NULL)
             errx(-1, "Unable to open interface %s: %s", options.intf2, ebuf);
 
         if ((eth_buff = sendpacket_get_hwaddr(sp)) == NULL) {
@@ -241,7 +230,7 @@ post_args(_U_ int argc, _U_ char *argv[])
             err(-1, "Please consult the man page for using the -M option.");
         }
         sendpacket_close(sp);
-        memcpy(options.intf2_mac, eth_buff, ETHER_ADDR_LEN);        
+        memcpy(options.intf2_mac, eth_buff, ETHER_ADDR_LEN);
     }
 
     /* 
@@ -260,7 +249,7 @@ post_args(_U_ int argc, _U_ char *argv[])
     if ((options.pcap2 = pcap_open_live(options.intf2, options.snaplen,
                                           options.promisc, options.to_ms, ebuf)) == NULL)
         errx(-1, "Unable to open interface %s: %s", options.intf2, ebuf);
-    
+
     /* poll should be -1 to wait indefinitely */
     options.poll_timeout = -1;
 }
