@@ -1,42 +1,31 @@
-/* $Id: edit_packet.c 2423 2010-03-13 07:09:49Z aturner $ */
+/* $Id$ */
 
 /*
- * Copyright (c) 2001-2010 Aaron Turner.
- * All rights reserved.
+ *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
+ *   Copyright (c) 2013-2014 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *   The Tcpreplay Suite of tools is free software: you can redistribute it 
+ *   and/or modify it under the terms of the GNU General Public License as 
+ *   published by the Free Software Foundation, either version 3 of the 
+ *   License, or with the authors permission any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the names of the copyright owners nor the names of its
- *    contributors may be used to endorse or promote products derived from
- *    this software without specific prior written permission.
+ *   The Tcpreplay Suite is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
- * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *   You should have received a copy of the GNU General Public License
+ *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "defines.h"
 #include "common.h"
 
-#include "tcpedit-int.h"
+#include "tcpedit.h"
 #include "edit_packet.h"
 #include "checksum.h"
+#include "incremental_checksum.h"
 #include "lib/sll.h"
 #include "dlt.h"
 
@@ -47,9 +36,9 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-static u_int32_t randomize_ipv4_addr(tcpedit_t *tcpedit, u_int32_t ip);
-static u_int32_t remap_ipv4(tcpedit_t *tcpedit, tcpr_cidr_t *cidr, const u_int32_t original);
-static int is_unicast_ipv4(tcpedit_t *tcpedit, u_int32_t ip);
+static uint32_t randomize_ipv4_addr(tcpedit_t *tcpedit, uint32_t ip);
+static uint32_t remap_ipv4(tcpedit_t *tcpedit, tcpr_cidr_t *cidr, const uint32_t original);
+static int is_unicast_ipv4(tcpedit_t *tcpedit, uint32_t ip);
 
 static void randomize_ipv6_addr(tcpedit_t *tcpedit, struct tcpr_in6_addr *addr);
 static int remap_ipv6(tcpedit_t *tcpedit, tcpr_cidr_t *cidr, struct tcpr_in6_addr *addr);
@@ -117,26 +106,134 @@ fix_ipv6_checksums(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr, ipv6_hdr_t *i
     return TCPEDIT_OK;
 }
 
+static void ipv4_l34_csum_replace(uint8_t *data, uint8_t protocol,
+        uint32_t old, uint32_t new)
+{
+    ipv4_hdr_t *ipv4;
+    tcp_hdr_t *tcp_hdr;
+    udp_hdr_t *udp_hdr;
+
+    assert(data);
+
+    switch (protocol) {
+    case IPPROTO_IP:
+        ipv4 = (ipv4_hdr_t *)data;
+        csum_replace4(&ipv4->ip_sum, old, new);
+        break;
+
+    case IPPROTO_TCP:
+        tcp_hdr = (tcp_hdr_t *)data;
+        csum_replace4(&tcp_hdr->th_sum, old, new);
+        break;
+
+    case IPPROTO_UDP:
+        udp_hdr = (udp_hdr_t *)data;
+        if (udp_hdr->uh_sum)
+            csum_replace4(&udp_hdr->uh_sum, old, new);
+        break;
+
+    default:
+        assert(false);
+    }
+
+}
+
+static void ipv6_l34_csum_replace(uint8_t *data, uint8_t protocol,
+        uint32_t *old, uint32_t *new)
+{
+    tcp_hdr_t *tcp_hdr;
+    udp_hdr_t *udp_hdr;
+    icmpv4_hdr_t *icmp;
+    icmpv6_hdr_t *icmp6;
+
+    assert(data);
+
+    switch (protocol) {
+
+    case IPPROTO_TCP:
+        tcp_hdr = (tcp_hdr_t *)data;
+        csum_replace16(&tcp_hdr->th_sum, old, new);
+        break;
+
+    case IPPROTO_UDP:
+        udp_hdr = (udp_hdr_t *)data;
+        if (udp_hdr->uh_sum)
+            csum_replace16(&udp_hdr->uh_sum, old, new);
+        break;
+
+    case IPPROTO_ICMP:
+        icmp = (icmpv4_hdr_t *)data;
+        csum_replace16(&icmp->icmp_sum, old, new);
+        break;
+
+    case IPPROTO_ICMP6:
+        icmp6 = (icmpv6_hdr_t *)data;
+        csum_replace16(&icmp6->icmp_sum, old, new);
+        break;
+
+    default:
+        assert(false);
+    }
+
+}
+
+static void ipv4_addr_csum_replace(ipv4_hdr_t *ip_hdr, uint32_t old_ip, uint32_t new_ip)
+{
+    uint8_t *l4 = NULL, protocol;
+    assert(ip_hdr);
+
+    ipv4_l34_csum_replace((uint8_t*)ip_hdr, IPPROTO_IP, old_ip, new_ip);
+
+      protocol = ip_hdr->ip_p;
+    if (protocol == IPPROTO_TCP || protocol == IPPROTO_UDP)
+        l4 = get_layer4_v4(ip_hdr, 65536);
+
+    if (!l4)
+        return;
+
+    /* if this is a fragment, don't attempt to checksum the Layer4 header */
+    if ((htons(ip_hdr->ip_off) & IP_OFFMASK) == 0)
+        ipv4_l34_csum_replace(l4, protocol, old_ip, new_ip);
+}
+
+static void ipv6_addr_csum_replace(ipv6_hdr_t *ip6_hdr,
+        struct tcpr_in6_addr *old_ip, struct tcpr_in6_addr *new_ip)
+{
+    uint8_t *l4 = NULL, protocol;
+    assert(ip6_hdr);
+
+      protocol = get_ipv6_l4proto(ip6_hdr, 65536);;
+    if (protocol == IPPROTO_TCP || protocol == IPPROTO_UDP ||
+            protocol == IPPROTO_ICMP || protocol == IPPROTO_ICMP6)
+        l4 = get_layer4_v6(ip6_hdr, 65536);
+
+    if (!l4)
+        return;
+
+    ipv6_l34_csum_replace(l4, protocol, (uint32_t*)old_ip, (uint32_t*)new_ip);
+}
+
+
 /**
  * returns a new 32bit integer which is the randomized IP 
  * based upon the user specified seed
  */
-static u_int32_t
-randomize_ipv4_addr(tcpedit_t *tcpedit, u_int32_t ip)
+static uint32_t
+randomize_ipv4_addr(tcpedit_t *tcpedit, uint32_t ip)
 {
     assert(tcpedit);
-    
+
     /* don't rewrite broadcast addresses */
     if (tcpedit->skip_broadcast && !is_unicast_ipv4(tcpedit, ip))
         return ip;
-        
+
     return ((ip ^ htonl(tcpedit->seed)) - (ip & htonl(tcpedit->seed)));
 }
 
 static void
 randomize_ipv6_addr(tcpedit_t *tcpedit, struct tcpr_in6_addr *addr)
 {
-    u_int32_t *p;
+    uint32_t *p;
     int i;
     u_char was_multicast;
 
@@ -185,13 +282,17 @@ randomize_ipv4(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
 
     /* don't rewrite broadcast addresses */
     if ((tcpedit->skip_broadcast && is_unicast_ipv4(tcpedit, (u_int32_t)ip_hdr->ip_dst.s_addr)) 
-        || !tcpedit->skip_broadcast) {
+            || !tcpedit->skip_broadcast) {
+        uint32_t old_ip = ip_hdr->ip_dst.s_addr;
         ip_hdr->ip_dst.s_addr = randomize_ipv4_addr(tcpedit, ip_hdr->ip_dst.s_addr);
+        ipv4_addr_csum_replace(ip_hdr, old_ip, ip_hdr->ip_dst.s_addr);
     }
     
     if ((tcpedit->skip_broadcast && is_unicast_ipv4(tcpedit, (u_int32_t)ip_hdr->ip_src.s_addr))
-        || !tcpedit->skip_broadcast) {
+            || !tcpedit->skip_broadcast) {
+        uint32_t old_ip = ip_hdr->ip_src.s_addr;
         ip_hdr->ip_src.s_addr = randomize_ipv4_addr(tcpedit, ip_hdr->ip_src.s_addr);
+        ipv4_addr_csum_replace(ip_hdr, old_ip, ip_hdr->ip_src.s_addr);
     }
 
 #ifdef DEBUG    
@@ -201,7 +302,7 @@ randomize_ipv4(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
 
     dbgx(1, "New Src IP: %s\tNew Dst IP: %s\n", srcip, dstip);
 
-    return(1);
+    return 0;
 }
 
 int
@@ -226,13 +327,19 @@ randomize_ipv6(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
 
     /* don't rewrite broadcast addresses */
     if ((tcpedit->skip_broadcast && !is_multicast_ipv6(tcpedit, &ip6_hdr->ip_dst))
-        || !tcpedit->skip_broadcast) {
+            || !tcpedit->skip_broadcast) {
+        struct tcpr_in6_addr old_ip6;
+        memcpy(&old_ip6, &ip6_hdr->ip_dst, sizeof(old_ip6));
         randomize_ipv6_addr(tcpedit, &ip6_hdr->ip_dst);
+        ipv6_addr_csum_replace(ip6_hdr, &old_ip6, &ip6_hdr->ip_dst);
     }
 
     if ((tcpedit->skip_broadcast && !is_multicast_ipv6(tcpedit, &ip6_hdr->ip_src))
-        || !tcpedit->skip_broadcast) {
+            || !tcpedit->skip_broadcast) {
+        struct tcpr_in6_addr old_ip6;
+        memcpy(&old_ip6, &ip6_hdr->ip_src, sizeof(old_ip6));
         randomize_ipv6_addr(tcpedit, &ip6_hdr->ip_src);
+        ipv6_addr_csum_replace(ip6_hdr, &old_ip6, &ip6_hdr->ip_src);
     }
 
 #ifdef DEBUG
@@ -242,7 +349,7 @@ randomize_ipv6(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
 
     dbgx(1, "New Src IP: %s\tNew Dst IP: %s\n", srcip, dstip);
 
-    return(1);
+    return 0;
 }
 
 /**
@@ -253,34 +360,61 @@ randomize_ipv6(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
 
 int
 untrunc_packet(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr, 
-        u_char *pktdata, ipv4_hdr_t *ip_hdr, ipv6_hdr_t *ip6_hdr)
+        u_char **pktdata, ipv4_hdr_t *ip_hdr, ipv6_hdr_t *ip6_hdr)
 {
     int l2len;
+    int chksum = 1;
+    u_char *packet;
+    udp_hdr_t *udp_hdr;
+    u_char *dataptr = NULL;
+
     assert(tcpedit);
     assert(pkthdr);
     assert(pktdata);
 
+    packet = *pktdata;
+    assert(packet);
+
     /* if actual len == cap len or there's no IP header, don't do anything */
     if ((pkthdr->caplen == pkthdr->len) || (ip_hdr == NULL && ip6_hdr == NULL)) {
         /* unless we're in MTU truncate mode */
         if (! tcpedit->mtu_truncate)
             return(0);
     }
-
+    
     if ((l2len = layer2len(tcpedit)) < 0) {
         tcpedit_seterr(tcpedit, "Non-sensical layer 2 length: %d", l2len);
         return -1;
     }
 
+    /*
+     * cannot checksum fragments, but we can do some
+     * work on UDP fragments. Setting checksum to 0
+     * means checksum will be ignored.
+     */
+    if (ip_hdr) {
+        if ((htons(ip_hdr->ip_off) & IP_OFFMASK) != 0) {
+            chksum = 0;
+        } else if (ip_hdr->ip_p == IPPROTO_UDP &&
+                (htons(ip_hdr->ip_off) & IP_MF) != 0) {
+            dataptr = (u_char*)ip_hdr;
+            dataptr += ip_hdr->ip_hl << 2;
+            udp_hdr = (udp_hdr_t*)dataptr;
+            udp_hdr->uh_sum = 0;
+            chksum = 0;
+        }
+    }
+
     /* Pad packet or truncate it */
     if (tcpedit->fixlen == TCPEDIT_FIXLEN_PAD) {
         /*
          * this should be an unnecessary check
-         * but I've gotten a report that sometimes the caplen > len
-         * which seems like a corrupted pcap
-         */
+           * but I've gotten a report that sometimes the caplen > len
+           * which seems like a corrupted pcap
+           */
         if (pkthdr->len > pkthdr->caplen) {
-            memset(pktdata + pkthdr->caplen, '\0', pkthdr->len - pkthdr->caplen);
+            packet = safe_realloc(packet, pkthdr->len);
+            memset(packet + pkthdr->caplen, '\0', pkthdr->len - pkthdr->caplen);
             pkthdr->caplen = pkthdr->len;
         } else if (pkthdr->len < pkthdr->caplen) {
             /* i guess this is necessary if we've got a bogus pcap */
@@ -295,10 +429,10 @@ untrunc_packet(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
         pkthdr->len = pkthdr->caplen;
     }
     else if (tcpedit->mtu_truncate) {
-        if (pkthdr->len > (u_int16_t)(tcpedit->mtu + l2len)) {
+        if (pkthdr->len > (uint32_t)(tcpedit->mtu + l2len)) {
             /* first truncate the packet */
             pkthdr->len = pkthdr->caplen = l2len + tcpedit->mtu;
-
+            
             /* if ip_hdr exists, update the length */
             if (ip_hdr != NULL) {
                 ip_hdr->ip_len = htons(tcpedit->mtu);
@@ -315,7 +449,7 @@ untrunc_packet(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
         return -1;
     }
 
-    return(1);
+    return chksum;
 }
 
 /**
@@ -330,7 +464,6 @@ extract_data(tcpedit_t *tcpedit, const u_char *pktdata, int caplen,
     int datalen = 0; /* amount of data beyond ip header */
     ipv4_hdr_t *ip_hdr = NULL;
     tcp_hdr_t *tcp_hdr = NULL;
-    udp_hdr_t *udp_hdr = NULL;
     u_char ipbuff[MAXPACKET];
     u_char *dataptr = NULL;
     
@@ -362,7 +495,7 @@ extract_data(tcpedit_t *tcpedit, const u_char *pktdata, int caplen,
 
     /* TCP ? */
     if (ip_hdr->ip_p == IPPROTO_TCP) {
-        tcp_hdr = (tcp_hdr_t *) get_layer4_v4(ip_hdr);
+        tcp_hdr = (tcp_hdr_t *) get_layer4_v4(ip_hdr, datalen);
         datalen -= tcp_hdr->th_off << 2;
         if (datalen <= 0)
             goto nodata;
@@ -372,7 +505,6 @@ extract_data(tcpedit_t *tcpedit, const u_char *pktdata, int caplen,
 
     /* UDP ? */
     else if (ip_hdr->ip_p == IPPROTO_UDP) {
-        udp_hdr = (udp_hdr_t *) get_layer4_v4(ip_hdr);
         datalen -= TCPR_UDP_H;
         if (datalen <= 0)
             goto nodata;
@@ -408,26 +540,29 @@ extract_data(tcpedit_t *tcpedit, const u_char *pktdata, int caplen,
 int
 rewrite_ipv4_ttl(tcpedit_t *tcpedit, ipv4_hdr_t *ip_hdr)
 {
+    uint16_t oldval, newval;
+
     assert(tcpedit);
 
     /* make sure there's something to edit */
-    if (ip_hdr == NULL || tcpedit->ttl_mode == TCPEDIT_TTL_OFF)
+    if (ip_hdr == NULL || tcpedit->ttl_mode == false)
         return(0);
         
+    oldval = (uint16_t)ip_hdr->ip_ttl;
     switch(tcpedit->ttl_mode) {
-    case TCPEDIT_TTL_SET:
+    case TCPEDIT_TTL_MODE_SET:
         if (ip_hdr->ip_ttl == tcpedit->ttl_value)
-            return(0);           /* no change required */
+            return 0;           /* no change required */
         ip_hdr->ip_ttl = tcpedit->ttl_value;
         break;
-    case TCPEDIT_TTL_ADD:
+    case TCPEDIT_TTL_MODE_ADD:
         if (((int)ip_hdr->ip_ttl + tcpedit->ttl_value) > 255) {
             ip_hdr->ip_ttl = 255;
         } else {
             ip_hdr->ip_ttl += tcpedit->ttl_value;
         }
         break;
-    case TCPEDIT_TTL_SUB:
+    case TCPEDIT_TTL_MODE_SUB:
         if (ip_hdr->ip_ttl <= tcpedit->ttl_value) {
             ip_hdr->ip_ttl = 1;
         } else {
@@ -437,7 +572,11 @@ rewrite_ipv4_ttl(tcpedit_t *tcpedit, ipv4_hdr_t *ip_hdr)
     default:
         errx(1, "invalid ttl_mode: %d", tcpedit->ttl_mode);
     }
-    return(1);
+
+    newval = (uint16_t)ip_hdr->ip_ttl;
+    csum_replace2(&ip_hdr->ip_sum, oldval, newval);
+
+    return 0;
 }
 
 /**
@@ -450,23 +589,23 @@ rewrite_ipv6_hlim(tcpedit_t *tcpedit, ipv6_hdr_t *ip6_hdr)
     assert(tcpedit);
 
     /* make sure there's something to edit */
-    if (ip6_hdr == NULL || tcpedit->ttl_mode == TCPEDIT_TTL_OFF)
+    if (ip6_hdr == NULL || tcpedit->ttl_mode == TCPEDIT_TTL_MODE_OFF)
         return(0);
 
     switch(tcpedit->ttl_mode) {
-    case TCPEDIT_TTL_SET:
+    case TCPEDIT_TTL_MODE_SET:
         if (ip6_hdr->ip_hl == tcpedit->ttl_value)
             return(0);           /* no change required */
         ip6_hdr->ip_hl = tcpedit->ttl_value;
         break;
-    case TCPEDIT_TTL_ADD:
+    case TCPEDIT_TTL_MODE_ADD:
         if (((int)ip6_hdr->ip_hl + tcpedit->ttl_value) > 255) {
             ip6_hdr->ip_hl = 255;
         } else {
             ip6_hdr->ip_hl += tcpedit->ttl_value;
         }
         break;
-    case TCPEDIT_TTL_SUB:
+    case TCPEDIT_TTL_MODE_SUB:
         if (ip6_hdr->ip_hl <= tcpedit->ttl_value) {
             ip6_hdr->ip_hl = 1;
         } else {
@@ -476,7 +615,7 @@ rewrite_ipv6_hlim(tcpedit_t *tcpedit, ipv6_hdr_t *ip6_hdr)
     default:
         errx(1, "invalid ttl_mode: %d", tcpedit->ttl_mode);
     }
-    return(1);
+    return 0;
 }
 
 /**
@@ -484,10 +623,10 @@ rewrite_ipv6_hlim(tcpedit_t *tcpedit, ipv6_hdr_t *ip6_hdr)
  * onto that netblock.  ie: 10.0.0.0/8 and 192.168.55.123 -> 10.168.55.123
  * while 10.150.9.0/24 and 192.168.55.123 -> 10.150.9.123
  */
-static u_int32_t
-remap_ipv4(tcpedit_t *tcpedit, tcpr_cidr_t *cidr, const u_int32_t original)
+static uint32_t
+remap_ipv4(tcpedit_t *tcpedit, tcpr_cidr_t *cidr, const uint32_t original)
 {
-    u_int32_t ipaddr = 0, network = 0, mask = 0, result = 0;
+    uint32_t ipaddr = 0, network = 0, mask = 0, result = 0;
 
     assert(tcpedit);
     assert(cidr);
@@ -563,23 +702,34 @@ rewrite_ipv4l3(tcpedit_t *tcpedit, ipv4_hdr_t *ip_hdr, tcpr_dir_t direction)
 {
     tcpr_cidrmap_t *cidrmap1 = NULL, *cidrmap2 = NULL;
     int didsrc = 0, diddst = 0, loop = 1;
+    tcpr_cidrmap_t *ipmap;
 
     assert(tcpedit);
     assert(ip_hdr);
 
     /* first check the src/dst IP maps */
-    if (tcpedit->srcipmap != NULL) {
-        if (ip_in_cidr(tcpedit->srcipmap->from, ip_hdr->ip_src.s_addr)) {
-            ip_hdr->ip_src.s_addr = remap_ipv4(tcpedit, tcpedit->srcipmap->to, ip_hdr->ip_src.s_addr);
+    ipmap = tcpedit->srcipmap;
+    while (ipmap != NULL) {
+        if (ip_in_cidr(ipmap->from, ip_hdr->ip_src.s_addr)) {
+            uint32_t old_ip = ip_hdr->ip_src.s_addr;
+            ip_hdr->ip_src.s_addr = remap_ipv4(tcpedit, ipmap->to, ip_hdr->ip_src.s_addr);
+            ipv4_addr_csum_replace(ip_hdr, old_ip, ip_hdr->ip_src.s_addr);
             dbgx(2, "Remapped src addr to: %s", get_addr2name4(ip_hdr->ip_src.s_addr, RESOLVE));
+            break;
         }
+        ipmap = ipmap->next;
     }
 
-    if (tcpedit->dstipmap != NULL) {
-        if (ip_in_cidr(tcpedit->dstipmap->from, ip_hdr->ip_dst.s_addr)) {
-            ip_hdr->ip_dst.s_addr = remap_ipv4(tcpedit, tcpedit->dstipmap->to, ip_hdr->ip_dst.s_addr);
-            dbgx(2, "Remapped src addr to: %s", get_addr2name4(ip_hdr->ip_dst.s_addr, RESOLVE));
+    ipmap = tcpedit->dstipmap;
+    while (ipmap != NULL) {
+        if (ip_in_cidr(ipmap->from, ip_hdr->ip_dst.s_addr)) {
+            uint32_t old_ip = ip_hdr->ip_dst.s_addr;
+            ip_hdr->ip_dst.s_addr = remap_ipv4(tcpedit, ipmap->to, ip_hdr->ip_dst.s_addr);
+            ipv4_addr_csum_replace(ip_hdr, old_ip, ip_hdr->ip_dst.s_addr);
+            dbgx(2, "Remapped dst addr to: %s", get_addr2name4(ip_hdr->ip_dst.s_addr, RESOLVE));
+            break;
         }
+        ipmap = ipmap->next;
     }
 
     /* anything else to rewrite? */
@@ -599,12 +749,16 @@ rewrite_ipv4l3(tcpedit_t *tcpedit, ipv4_hdr_t *ip_hdr, tcpr_dir_t direction)
     /* loop through the cidrmap to rewrite */
     do {
         if ((! diddst) && ip_in_cidr(cidrmap2->from, ip_hdr->ip_dst.s_addr)) {
+            uint32_t old_ip = ip_hdr->ip_dst.s_addr;
             ip_hdr->ip_dst.s_addr = remap_ipv4(tcpedit, cidrmap2->to, ip_hdr->ip_dst.s_addr);
+            ipv4_addr_csum_replace(ip_hdr, old_ip, ip_hdr->ip_dst.s_addr);
             dbgx(2, "Remapped dst addr to: %s", get_addr2name4(ip_hdr->ip_dst.s_addr, RESOLVE));
             diddst = 1;
         }
         if ((! didsrc) && ip_in_cidr(cidrmap1->from, ip_hdr->ip_src.s_addr)) {
+            uint32_t old_ip = ip_hdr->ip_src.s_addr;
             ip_hdr->ip_src.s_addr = remap_ipv4(tcpedit, cidrmap1->to, ip_hdr->ip_src.s_addr);
+            ipv4_addr_csum_replace(ip_hdr, old_ip, ip_hdr->ip_src.s_addr);
             dbgx(2, "Remapped src addr to: %s", get_addr2name4(ip_hdr->ip_src.s_addr, RESOLVE));
             didsrc = 1;
         }
@@ -634,8 +788,8 @@ rewrite_ipv4l3(tcpedit_t *tcpedit, ipv4_hdr_t *ip_hdr, tcpr_dir_t direction)
 
     } while (loop);
 
-    /* return how many changes we made */
-    return (diddst + didsrc);
+    /* return how many changes we require checksum updates (none) */
+    return 0;
 }
 
 int
@@ -643,23 +797,36 @@ rewrite_ipv6l3(tcpedit_t *tcpedit, ipv6_hdr_t *ip6_hdr, tcpr_dir_t direction)
 {
     tcpr_cidrmap_t *cidrmap1 = NULL, *cidrmap2 = NULL;
     int didsrc = 0, diddst = 0, loop = 1;
+    tcpr_cidrmap_t *ipmap;
 
     assert(tcpedit);
     assert(ip6_hdr);
 
     /* first check the src/dst IP maps */
-    if (tcpedit->srcipmap != NULL) {
-        if (ip6_in_cidr(tcpedit->srcipmap->from, &ip6_hdr->ip_src)) {
-            remap_ipv6(tcpedit, tcpedit->srcipmap->to, &ip6_hdr->ip_src);
+    ipmap = tcpedit->srcipmap;
+    while (ipmap != NULL) {
+        if (ip6_in_cidr(ipmap->from, &ip6_hdr->ip_src)) {
+            struct tcpr_in6_addr old_ip6;
+            memcpy(&old_ip6, &ip6_hdr->ip_src, sizeof(old_ip6));
+            remap_ipv6(tcpedit, ipmap->to, &ip6_hdr->ip_src);
+            ipv6_addr_csum_replace(ip6_hdr, &old_ip6, &ip6_hdr->ip_src);
             dbgx(2, "Remapped src addr to: %s", get_addr2name6(&ip6_hdr->ip_src, RESOLVE));
+            break;
         }
+        ipmap = ipmap->next;
     }
 
-    if (tcpedit->dstipmap != NULL) {
-        if (ip6_in_cidr(tcpedit->dstipmap->from, &ip6_hdr->ip_dst)) {
-            remap_ipv6(tcpedit, tcpedit->dstipmap->to, &ip6_hdr->ip_dst);
-            dbgx(2, "Remapped src addr to: %s", get_addr2name6(&ip6_hdr->ip_dst, RESOLVE));
+    ipmap = tcpedit->dstipmap;
+    while (ipmap != NULL) {
+        if (ip6_in_cidr(ipmap->from, &ip6_hdr->ip_dst)) {
+            struct tcpr_in6_addr old_ip6;
+            memcpy(&old_ip6, &ip6_hdr->ip_dst, sizeof(old_ip6));
+            remap_ipv6(tcpedit, ipmap->to, &ip6_hdr->ip_dst);
+            ipv6_addr_csum_replace(ip6_hdr, &old_ip6, &ip6_hdr->ip_dst);
+            dbgx(2, "Remapped dst addr to: %s", get_addr2name6(&ip6_hdr->ip_dst, RESOLVE));
+            break;
         }
+        ipmap = ipmap->next;
     }
 
     /* anything else to rewrite? */
@@ -679,12 +846,18 @@ rewrite_ipv6l3(tcpedit_t *tcpedit, ipv6_hdr_t *ip6_hdr, tcpr_dir_t direction)
     /* loop through the cidrmap to rewrite */
     do {
         if ((! diddst) && ip6_in_cidr(cidrmap2->from, &ip6_hdr->ip_dst)) {
+            struct tcpr_in6_addr old_ip6;
+            memcpy(&old_ip6, &ip6_hdr->ip_dst, sizeof(old_ip6));
             remap_ipv6(tcpedit, cidrmap2->to, &ip6_hdr->ip_dst);
+            ipv6_addr_csum_replace(ip6_hdr, &old_ip6, &ip6_hdr->ip_src);
             dbgx(2, "Remapped dst addr to: %s", get_addr2name6(&ip6_hdr->ip_dst, RESOLVE));
             diddst = 1;
         }
         if ((! didsrc) && ip6_in_cidr(cidrmap1->from, &ip6_hdr->ip_src)) {
+            struct tcpr_in6_addr old_ip6;
+            memcpy(&old_ip6, &ip6_hdr->ip_src, sizeof(old_ip6));
             remap_ipv6(tcpedit, cidrmap1->to, &ip6_hdr->ip_src);
+            ipv6_addr_csum_replace(ip6_hdr, &old_ip6, &ip6_hdr->ip_src);
             dbgx(2, "Remapped src addr to: %s", get_addr2name6(&ip6_hdr->ip_src, RESOLVE));
             didsrc = 1;
         }
@@ -714,8 +887,8 @@ rewrite_ipv6l3(tcpedit_t *tcpedit, ipv6_hdr_t *ip6_hdr, tcpr_dir_t direction)
 
     } while (loop);
 
-    /* return how many changes we made */
-    return (diddst + didsrc);
+    /* return how many changes we made (none) */
+    return 0;
 }
 
 /**
@@ -728,7 +901,7 @@ randomize_iparp(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
 {
     arp_hdr_t *arp_hdr = NULL;
     int l2len = 0;
-    u_int32_t *ip, tempip;
+    uint32_t *ip, tempip;
     u_char *add_hdr;
 
     assert(tcpedit);
@@ -748,14 +921,14 @@ randomize_iparp(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
         /* jump to the addresses */
         add_hdr = (u_char *)arp_hdr;
         add_hdr += sizeof(arp_hdr_t) + arp_hdr->ar_hln;
-        ip = (u_int32_t *)add_hdr;
+        ip = (uint32_t *)add_hdr;
         tempip = randomize_ipv4_addr(tcpedit, *ip);
-        memcpy(ip, &tempip, sizeof(u_int32_t));
+        memcpy(ip, &tempip, sizeof(uint32_t));
 
         add_hdr += arp_hdr->ar_pln + arp_hdr->ar_hln;
-        ip = (u_int32_t *)add_hdr;
+        ip = (uint32_t *)add_hdr;
         tempip = randomize_ipv4_addr(tcpedit, *ip);
-        memcpy(ip, &tempip, sizeof(u_int32_t));
+        memcpy(ip, &tempip, sizeof(uint32_t));
     }
 
     return 1; /* yes we changed the packet */
@@ -773,12 +946,12 @@ int
 rewrite_iparp(tcpedit_t *tcpedit, arp_hdr_t *arp_hdr, int cache_mode)
 {
     u_char *add_hdr = NULL;
-    u_int32_t *ip1 = NULL, *ip2 = NULL;
-    u_int32_t newip = 0;
+    uint32_t *ip1 = NULL, *ip2 = NULL;
+    uint32_t newip = 0;
     tcpr_cidrmap_t *cidrmap1 = NULL, *cidrmap2 = NULL;
     int didsrc = 0, diddst = 0, loop = 1;
 #ifdef FORCE_ALIGN
-    u_int32_t iptemp;
+    uint32_t iptemp;
 #endif
 
     assert(tcpedit);
@@ -809,14 +982,14 @@ rewrite_iparp(tcpedit_t *tcpedit, arp_hdr_t *arp_hdr, int cache_mode)
         /* jump to the addresses */
         add_hdr = (u_char *)arp_hdr;
         add_hdr += sizeof(arp_hdr_t) + arp_hdr->ar_hln;
-        ip1 = (u_int32_t *)add_hdr;
+        ip1 = (uint32_t *)add_hdr;
         add_hdr += arp_hdr->ar_pln + arp_hdr->ar_hln;
 #ifdef FORCE_ALIGN
         /* copy IP2 to a temporary buffer for processing */
-        memcpy(&iptemp, add_hdr, sizeof(u_int32_t));
+        memcpy(&iptemp, add_hdr, sizeof(uint32_t));
         ip2 = &iptemp;
 #else
-        ip2 = (u_int32_t *)add_hdr;
+        ip2 = (uint32_t *)add_hdr;
 #endif
         
 
@@ -851,7 +1024,7 @@ rewrite_iparp(tcpedit_t *tcpedit, arp_hdr_t *arp_hdr, int cache_mode)
 
 #ifdef FORCE_ALIGN
             /* copy temporary IP to IP2 location in buffer */
-            memcpy(add_hdr, &iptemp, sizeof(u_int32_t));
+            memcpy(add_hdr, &iptemp, sizeof(uint32_t));
 #endif
 
             /*
@@ -886,12 +1059,12 @@ rewrite_iparp(tcpedit_t *tcpedit, arp_hdr_t *arp_hdr, int cache_mode)
  * for broadcast/multicast addresses.  Returns -1 on error
  */
 static int
-is_unicast_ipv4(tcpedit_t *tcpedit, u_int32_t ip)
+is_unicast_ipv4(tcpedit_t *tcpedit, uint32_t ip)
 {
     assert(tcpedit);
    
     /* multicast/broadcast is 224.0.0.0 or greater */
-    if (ip > 3758096384)
+    if (ntohl(ip) > 3758096384)
         return 0;
         
     return 1;
