@@ -1,45 +1,33 @@
-/* $Id: ieee80211.c 2423 2010-03-13 07:09:49Z aturner $ */
+/* $Id$ */
 
 /*
- * Copyright (c) 2006-2010 Aaron Turner.
- * All rights reserved.
+ *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
+ *   Copyright (c) 2013-2014 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *   The Tcpreplay Suite of tools is free software: you can redistribute it 
+ *   and/or modify it under the terms of the GNU General Public License as 
+ *   published by the Free Software Foundation, either version 3 of the 
+ *   License, or with the authors permission any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the names of the copyright owners nor the names of its
- *    contributors may be used to endorse or promote products derived from
- *    this software without specific prior written permission.
+ *   The Tcpreplay Suite is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
- * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *   You should have received a copy of the GNU General Public License
+ *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
 #include <string.h>
 
-#include "dlt_plugins-int.h"
-#include "dlt_utils.h"
-#include "ieee80211.h"
-#include "ieee80211_hdr.h"
 #include "tcpedit.h"
 #include "common.h"
 #include "tcpr.h"
+#include "dlt_utils.h"
+#include "tcpedit_stub.h"
+#include "ieee80211.h"
+#include "ieee80211_hdr.h"
 
 /*
  * Notes about the ieee80211 plugin:
@@ -50,7 +38,7 @@
  */
 static char dlt_name[] = "ieee80211";
 _U_ static char dlt_prefix[] = "ieee802_11";
-static u_int16_t dlt_value = DLT_IEEE802_11;
+static uint16_t dlt_value = DLT_IEEE802_11;
 
 /*
  * Function to register ourselves.  This function is always called, regardless
@@ -113,7 +101,6 @@ int
 dlt_ieee80211_init(tcpeditdlt_t *ctx)
 {
     tcpeditdlt_plugin_t *plugin;
-    ieee80211_config_t *config;
     assert(ctx);
     
     if ((plugin = tcpedit_dlt_getplugin(ctx, dlt_value)) == NULL) {
@@ -128,9 +115,7 @@ dlt_ieee80211_init(tcpeditdlt_t *ctx)
     /* allocate memory for our config data */
     if (sizeof(ieee80211_config_t) > 0)
         plugin->config = safe_malloc(sizeof(ieee80211_config_t));
-    
-    config = (ieee80211_config_t *)plugin->config;
-    
+
     /* FIXME: set default config values here */
 
     return TCPEDIT_OK; /* success */
@@ -242,7 +227,7 @@ dlt_ieee80211_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
 {
     int l2len;
     int hdrlen = 0;
-    u_int16_t *frame_control, fc;
+    uint16_t *frame_control, fc;
     struct tcpr_802_2snap_hdr *hdr;
 
     assert(ctx);
@@ -252,7 +237,7 @@ dlt_ieee80211_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     assert(pktlen >= l2len);
 
     /* check 802.11 frame control field */
-    frame_control = (u_int16_t *)packet;
+    frame_control = (uint16_t *)packet;
     fc = ntohs(*frame_control);
 
     /* Not all 802.11 frames have data */
@@ -331,7 +316,7 @@ dlt_ieee80211_merge_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen,
 int
 dlt_ieee80211_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
 {
-    u_int16_t *frame_control, fc;
+    uint16_t *frame_control, fc;
     struct tcpr_802_2snap_hdr *hdr;
     int hdrlen = 0;
 
@@ -342,7 +327,7 @@ dlt_ieee80211_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen)
     
     dbgx(2, "packet = %p\t\tplen = %d", packet, pktlen);
 
-    frame_control = (u_int16_t *)packet;
+    frame_control = (uint16_t *)packet;
     fc = ntohs(*frame_control);
 
     
@@ -386,7 +371,7 @@ dlt_ieee80211_get_mac(tcpeditdlt_t *ctx, tcpeditdlt_mac_type_t mac, const u_char
     assert(ctx);
     assert(packet);
     assert(pktlen);
-    char *macaddr;
+    u_char *macaddr;
     
     switch(mac) {
     case SRC_MAC:
