@@ -1,48 +1,36 @@
-/* $Id: user.c 2423 2010-03-13 07:09:49Z aturner $ */
+/* $Id$ */
 
 /*
- * Copyright (c) 2006-2010 Aaron Turner.
- * All rights reserved.
+ *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
+ *   Copyright (c) 2013-2014 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *   The Tcpreplay Suite of tools is free software: you can redistribute it 
+ *   and/or modify it under the terms of the GNU General Public License as 
+ *   published by the Free Software Foundation, either version 3 of the 
+ *   License, or with the authors permission any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the names of the copyright owners nor the names of its
- *    contributors may be used to endorse or promote products derived from
- *    this software without specific prior written permission.
+ *   The Tcpreplay Suite is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
- * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *   You should have received a copy of the GNU General Public License
+ *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
 #include <string.h>
 
-#include "dlt_plugins-int.h"
-#include "dlt_utils.h"
-#include "user.h"
 #include "tcpedit.h"
 #include "common.h"
 #include "tcpr.h"
+#include "dlt_utils.h"
+#include "tcpedit_stub.h"
+#include "user.h"
 
 static char dlt_name[] = "user";
 static char _U_ dlt_prefix[] = "user";
-static u_int16_t dlt_value = DLT_USER0;
+static uint16_t dlt_value = DLT_USER0;
 
 /*
  * Function to register ourselves.  This function is always called, regardless
@@ -122,6 +110,8 @@ dlt_user_init(tcpeditdlt_t *ctx)
         plugin->config = safe_malloc(sizeof(user_config_t));
     
     config = (user_config_t *)plugin->config;
+    config->length = -1;
+    
     /* do nothing */
     return TCPEDIT_OK; /* success */
 }
@@ -185,7 +175,7 @@ dlt_user_parse_opts(tcpeditdlt_t *ctx)
     /* --user-dlink */
     if (HAVE_OPT(USER_DLINK)) {
         int  ct = STACKCT_OPT(USER_DLINK);
-        char **list = STACKLST_OPT(USER_DLINK);
+        char **list = (char**)STACKLST_OPT(USER_DLINK);
         int first = 1;
         
         do  {
@@ -360,7 +350,7 @@ dlt_user_l2addr_type(void)
  * Need this special function for dlt_plugins.c:tcpedit_dlt_output_dlt()
  */
 
-u_int16_t
+uint16_t
 dlt_user_get_output_dlt(tcpeditdlt_t *ctx)
 {
     tcpeditdlt_plugin_t *plugin;
