@@ -1,43 +1,31 @@
-/* $Id: dlt_utils.c 2423 2010-03-13 07:09:49Z aturner $ */
+/* $Id$ */
 
 /*
- * Copyright (c) 2006-2010 Aaron Turner.
- * All rights reserved.
+ *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
+ *   Copyright (c) 2013-2014 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *   The Tcpreplay Suite of tools is free software: you can redistribute it 
+ *   and/or modify it under the terms of the GNU General Public License as 
+ *   published by the Free Software Foundation, either version 3 of the 
+ *   License, or with the authors permission any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the names of the copyright owners nor the names of its
- *    contributors may be used to endorse or promote products derived from
- *    this software without specific prior written permission.
+ *   The Tcpreplay Suite is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
- * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *   You should have received a copy of the GNU General Public License
+ *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <string.h>
 
-#include "dlt_plugins-int.h"  
+#include "tcpedit.h"
 #include "dlt_utils.h"
 #include "common.h"
 
 /* from dlt_plugins.c */
-extern const u_int32_t tcpeditdlt_bit_map[];
+extern const uint32_t tcpeditdlt_bit_map[];
 extern const char *tcpeditdlt_bit_info[];
 
 /*
@@ -182,11 +170,11 @@ tcpedit_dlt_addplugin(tcpeditdlt_t *ctx, tcpeditdlt_plugin_t *new)
 int
 tcpedit_dlt_validate(tcpeditdlt_t *ctx)
 {
-    u_int32_t bit;
+    uint32_t bit;
     
     /* loops from 1 -> UINT32_MAX by powers of 2 */
     for (bit = 1; bit != 0; bit = bit << 2) {
-        if (ctx->encoder->requires & bit && ! ctx->decoder->provides & bit) {
+        if (ctx->encoder->requires & bit && !(ctx->decoder->provides & bit)) {
             tcpedit_seterr(ctx->tcpedit, "%s", tcpeditdlt_bit_info[tcpeditdlt_bit_map[bit]]);
             return TCPEDIT_ERROR;
         }            
@@ -262,3 +250,32 @@ tcpedit_dlt_l3data_merge(tcpeditdlt_t *ctx, u_char *packet, int pktlen, const u_
 #endif
     return packet;
 }
+
+/**
+ * When using subdecoders, we need to transfer the sub decoder state
+ * to our state so that our primary encoder has it available.
+ */
+int
+tcpedit_dlt_copy_decoder_state(tcpeditdlt_t *ctx, tcpeditdlt_t *subctx)
+{
+    assert(ctx);
+    assert(subctx);
+
+    memcpy(&ctx->srcaddr, &subctx->srcaddr, sizeof(ctx->srcaddr));
+    memcpy(&ctx->dstaddr, &subctx->dstaddr, sizeof(ctx->dstaddr));
+    ctx->proto = subctx->proto;
+    memcpy(&ctx->srcmac, &subctx->srcmac, MAX_MAC_LEN);
+    memcpy(&ctx->dstmac, &subctx->dstmac, MAX_MAC_LEN);
+    
+    /* just need to copy the ptr */
+    ctx->decoded_extra = subctx->decoded_extra;
+
+    /* 
+     * the first decoder should of alraedy specified it's l2len, so we need to
+     * add to it the l2len determined by the sub-plugin
+     */
+    ctx->l2len += subctx->l2len;
+    
+    return TCPEDIT_OK;
+}
+
