@@ -1,33 +1,21 @@
-/* $Id: tcpedit.c 2423 2010-03-13 07:09:49Z aturner $ */
+/* $Id$ */
 
 /*
- * Copyright (c) 2001-2010 Aaron Turner.
- * All rights reserved.
+ *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
+ *   Copyright (c) 2013-2014 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *   The Tcpreplay Suite of tools is free software: you can redistribute it 
+ *   and/or modify it under the terms of the GNU General Public License as 
+ *   published by the Free Software Foundation, either version 3 of the 
+ *   License, or with the authors permission any later version.
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the names of the copyright owners nor the names of its
- *    contributors may be used to endorse or promote products derived from
- *    this software without specific prior written permission.
+ *   The Tcpreplay Suite is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
- * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *   You should have received a copy of the GNU General Public License
+ *   along with the Tcpreplay Suite.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -42,13 +30,12 @@
 #include <unistd.h>
 #include <stdarg.h>
 
-#include "tcpedit-int.h"
 #include "tcpedit_stub.h"
 #include "portmap.h"
 #include "common.h"
+#include "incremental_checksum.h"
 #include "edit_packet.h"
 #include "parse_args.h"
-#include "plugins/dlt_plugins.h"
 
 
 #include "lib/sll.h"
@@ -56,6 +43,32 @@
 
 tOptDesc *const tcpedit_tcpedit_optDesc_p;
 
+/**
+ * \brief Checks to see if you should make an edit
+ *
+ * Given the packet direction, this lets you know if you should make an edit
+ *
+ * packet: C2S & editdir = client == 1
+ * packet: C2S & editdir = server == 0
+ * packet: S2C & editdir = client == 0
+ * packet: S2C & editdir = server == 1
+ * packet: S2C & editdir = both   == 1
+ * packet: C2S & editdir = both   == 1
+ */
+int
+tcpedit_checkdir(tcpedit_t *tcpedit, tcpr_dir_t direction)
+{
+
+    /* Should we edit this packet? */
+    if ((tcpedit->editdir == TCPEDIT_EDIT_BOTH) ||
+        (tcpedit->editdir == TCPEDIT_EDIT_C2S && direction == TCPR_DIR_C2S) ||
+        (tcpedit->editdir == TCPEDIT_EDIT_S2C && direction == TCPR_DIR_S2C)) {
+        return 1;
+    }
+    return 0;
+}
+
+
 /**
  * \brief Edit the given packet
  *
@@ -89,6 +102,7 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     dbgx(3, "packet " COUNTER_SPEC " caplen %d", 
             tcpedit->runtime.packetnum, (*pkthdr)->caplen);
 
+
     /*
      * remove the Ethernet FCS (checksum)?
      * note that this feature requires the end user to be smart and
@@ -131,13 +145,13 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
 
     /* does packet have an IP header?  if so set our pointer to it */
     if (l2proto == htons(ETHERTYPE_IP)) {
-        ip_hdr = (ipv4_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, src_dlt, packet, (*pkthdr)->caplen);
+        ip_hdr = (ipv4_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen);
         if (ip_hdr == NULL) {
             return TCPEDIT_ERROR;
         }        
         dbgx(3, "Packet has an IPv4 header: %p...", ip_hdr);
     } else if (l2proto == htons(ETHERTYPE_IP6)) {
-        ip6_hdr = (ipv6_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, src_dlt, packet, (*pkthdr)->caplen);
+        ip6_hdr = (ipv6_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen);
         if (ip6_hdr == NULL) {
             return TCPEDIT_ERROR;
         }
@@ -152,23 +166,26 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     if (ip_hdr != NULL) {
         
         /* set TOS ? */
-        if (tcpedit->tos > -1)
+        if (tcpedit->tos > -1) {
+            uint16_t newval, oldval = *((uint16_t*)ip_hdr);
             ip_hdr->ip_tos = tcpedit->tos;
+            newval = *((uint16_t*)ip_hdr);
+            csum_replace2(&ip_hdr->ip_sum, oldval, newval);
+        }
             
         /* rewrite the TTL */
-        needtorecalc += rewrite_ipv4_ttl(tcpedit, ip_hdr);
+        rewrite_ipv4_ttl(tcpedit, ip_hdr);
 
         /* rewrite TCP/UDP ports */
         if (tcpedit->portmap != NULL) {
             if ((retval = rewrite_ipv4_ports(tcpedit, &ip_hdr)) < 0)
                 return TCPEDIT_ERROR;
-            needtorecalc += retval;
         }
     }
     /* IPv6 edits */
     else if (ip6_hdr != NULL) {
         /* rewrite the hop limit */
-        needtorecalc += rewrite_ipv6_hlim(tcpedit, ip6_hdr);
+        rewrite_ipv6_hlim(tcpedit, ip6_hdr);
 
         /* set traffic class? */
         if (tcpedit->tclass > -1) {
@@ -200,13 +217,12 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         if (tcpedit->portmap != NULL) {
             if ((retval = rewrite_ipv6_ports(tcpedit, &ip6_hdr)) < 0)
                 return TCPEDIT_ERROR;
-            needtorecalc += retval;
         }
     }
 
     /* (Un)truncate or MTU truncate packet? */
     if (tcpedit->fixlen || tcpedit->mtu_truncate) {
-        if ((retval = untrunc_packet(tcpedit, *pkthdr, packet, ip_hdr, ip6_hdr)) < 0)
+        if ((retval = untrunc_packet(tcpedit, *pkthdr, pktdata, ip_hdr, ip6_hdr)) < 0)
             return TCPEDIT_ERROR;
         needtorecalc += retval;
     }
@@ -217,11 +233,9 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         if (ip_hdr != NULL) {
             if ((retval = rewrite_ipv4l3(tcpedit, ip_hdr, direction)) < 0)
                 return TCPEDIT_ERROR;
-            needtorecalc += retval;
         } else if (ip6_hdr != NULL) {
             if ((retval = rewrite_ipv6l3(tcpedit, ip6_hdr, direction)) < 0)
                 return TCPEDIT_ERROR;
-            needtorecalc += retval;
         }
 
         /* ARP packets */
@@ -244,13 +258,11 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
             if ((retval = randomize_ipv4(tcpedit, *pkthdr, packet, 
                     ip_hdr)) < 0)
                 return TCPEDIT_ERROR;
-            needtorecalc += retval;
 
         } else if (ip6_hdr != NULL) {
             if ((retval = randomize_ipv6(tcpedit, *pkthdr, packet,
                     ip6_hdr)) < 0)
                 return TCPEDIT_ERROR;
-            needtorecalc += retval;
 
         /* ARP packets */
         } else if (l2proto == htons(ETHERTYPE_ARP)) {
@@ -269,16 +281,19 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     /* do we need to fix checksums? -- must always do this last! */
     if ((tcpedit->fixcsum || needtorecalc)) {
         if (ip_hdr != NULL) {
+            dbgx(3, "doing IPv4 checksum: needtorecalc=%d", needtorecalc);
             retval = fix_ipv4_checksums(tcpedit, *pkthdr, ip_hdr);
         } else if (ip6_hdr != NULL) {
+            dbgx(3, "doing IPv6 checksum: needtorecalc=%d", needtorecalc);
             retval = fix_ipv6_checksums(tcpedit, *pkthdr, ip6_hdr);
         } else {
+            dbgx(3, "checksum not performed: needtorecalc=%d", needtorecalc);
             retval = TCPEDIT_OK;
         }
         if (retval < 0) {
             return TCPEDIT_ERROR;
         } else if (retval == TCPEDIT_WARN) {
-            warnx("%s", tcpedit_getwarn(tcpedit));
+            dbgx(3, "%s", tcpedit_getwarn(tcpedit));
         }
     }
 
@@ -309,6 +324,7 @@ tcpedit_init(tcpedit_t **tcpedit_ex, int dlt)
     tcpedit->tos = -1;
     tcpedit->tclass = -1;
     tcpedit->flowlabel = -1;
+    tcpedit->editdir = TCPEDIT_EDIT_BOTH;
  
     memset(&(tcpedit->runtime), 0, sizeof(tcpedit_runtime_t));
     tcpedit->runtime.dlt1 = dlt;
@@ -493,7 +509,7 @@ tcpedit_close(tcpedit_t *tcpedit)
  * Return a ptr to the Layer 3 data.  Returns TCPEDIT_ERROR on error
  */
 const u_char *
-tcpedit_l3data(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const int pktlen)
+tcpedit_l3data(tcpedit_t *tcpedit, tcpedit_coder code, u_char *packet, const int pktlen)
 {
     u_char *result = NULL;
     if (code == BEFORE_PROCESS) {
@@ -508,7 +524,7 @@ tcpedit_l3data(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const i
  * return the length of the layer 2 header.  Returns TCPEDIT_ERROR on error
  */
 int 
-tcpedit_l2len(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const int pktlen)
+tcpedit_l2len(tcpedit_t *tcpedit, tcpedit_coder code, u_char *packet, const int pktlen)
 {
     int result = 0;
     if (code == BEFORE_PROCESS) {
@@ -523,7 +539,7 @@ tcpedit_l2len(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const in
  * Returns the layer 3 type, often encoded as the layer2.proto field
  */
 int 
-tcpedit_l3proto(tcpedit_t *tcpedit, tcpedit_coder_t code, const u_char *packet, const int pktlen)
+tcpedit_l3proto(tcpedit_t *tcpedit, tcpedit_coder code, const u_char *packet, const int pktlen)
 {
     int result = 0;
     if (code == BEFORE_PROCESS) {
@@ -536,19 +552,19 @@ tcpedit_l3proto(tcpedit_t *tcpedit, tcpedit_coder_t code, const u_char *packet,
 
 /*
 u_char *
-tcpedit_srcmac(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const int pktlen)
+tcpedit_srcmac(tcpedit_t *tcpedit, tcpedit_coder code, u_char *packet, const int pktlen)
 {
    
 }
 
 u_char *
-tcpedit_dstmac(tcpedit_t *tcpedit, tcpedit_coder_t code, u_char *packet, const int pktlen)
+tcpedit_dstmac(tcpedit_t *tcpedit, tcpedit_coder code, u_char *packet, const int pktlen)
 {
     
 }
 
 int 
-tcpedit_maclen(tcpedit_t *tcpedit, tcpedit_coder_t code)
+tcpedit_maclen(tcpedit_t *tcpedit, tcpedit_coder code)
 {
     
 }
